@@ -351,9 +351,13 @@ public class ClassType extends ObjectType
   public void setName (String name)
   {
     this_name = name;
-    setSignature("L"+name.replace('.', '/')+";");
+    setSignature(nameToSignature(name));
   }
 
+    public static String nameToSignature(String name) {
+        return "L"+name.replace('.', '/')+";";
+    }
+
   SourceDebugExtAttr sourceDbgExt;
 
   /** Create a <code>SourceDebugExtAttr</code>, if needed, and
