@@ -62,10 +62,16 @@ public class ChainLambdas extends ExpExpVisitor<ScopeExp> {
         for (int i = 0; i < nargs;  i++) {
             e = visit(args[i], scope);
             if (e.neverReturns()) {
-                Expression[] xargs = new Expression[i+1];
+                Expression[] xargs = new Expression[i+2];
                 xargs[0] = exp.func;
-                System.arraycopy(args, 0, xargs, 1, i);
-                maybeWarnUnreachable(i+1 < nargs ? args[i+1] : exp);
+                System.arraycopy(args, 0, xargs, 1, i+1);
+                if (i+1 < nargs || ! exp.isAppendValues()) {
+                    if (! unreachableCodeSeen && comp.warnUnreachable()) {
+                        comp.error('w', "unreachable procedure call", exp);
+                        comp.error('i', "this operand never finishes", args[i]);
+                    }
+                    unreachableCodeSeen = true;
+                }
                 BeginExp bexp = new BeginExp(xargs);
                 bexp.type = Type.neverReturnsType;
                 return bexp;
@@ -75,6 +81,18 @@ public class ChainLambdas extends ExpExpVisitor<ScopeExp> {
         return exp;
     }
 
+    protected Expression visitSetExp(SetExp sexp, ScopeExp scope) {
+        Expression r = super.visitSetExp(sexp, scope);
+        if (r == sexp) {
+            Expression rval = sexp.getNewValue();
+            if (rval.neverReturns()) {
+                maybeWarnUnreachable(sexp);
+                return rval;
+            }
+        }
+        return r;
+    }
+
     protected Expression visitIfExp(IfExp exp, ScopeExp scope) {
         Expression e = visit(exp.test, scope);
         if (e.neverReturns()) {
