@@ -1079,7 +1079,8 @@ public class Compilation implements SourceLocator
   public boolean makeRunnable ()
   {
     return ! generatingServlet() && ! generatingApplet()
-      && ! getModule().staticInitRun();
+      && ! getModule().staticInitRun()
+      && ! getModule().getFlag(ModuleExp.USE_DEFINED_CLASS);
   }
 
   public void addMainClass (ModuleExp module)
@@ -1096,6 +1097,8 @@ public class Compilation implements SourceLocator
 	  sup = typeApplet;
 	else if (generatingServlet())
 	  sup = typeServlet;
+        else if (module.getFlag(ModuleExp.USE_DEFINED_CLASS))
+          sup = Type.objectType;
 	else
 	  sup = getModuleType();
       }
@@ -1158,7 +1161,8 @@ public class Compilation implements SourceLocator
     if (curClass == mainClass
         // Optimization: No point in calling ModuleInfo.register if we aren't
         // compiling a named module.
-        && minfo != null && minfo.sourcePath != null)
+        && minfo != null && minfo.sourcePath != null
+        && ! getModule().getFlag(ModuleExp.USE_DEFINED_CLASS))
       {
 	code.emitPushThis();
 	code.emitInvokeStatic(ClassType.make("gnu.expr.ModuleInfo")
@@ -1274,7 +1278,7 @@ public class Compilation implements SourceLocator
 	Type[] matchArgs = null;
 	for (int j = numApplyMethods;  --j >= 0; )
 	  {
-	    LambdaExp source = (LambdaExp) lexp.applyMethods.elementAt(j);
+	    LambdaExp source = lexp.applyMethods.get(j);
 	    // Select the subset of source.primMethods[*] that are suitable
 	    // for the current apply method.
 	    Method[] primMethods = source.primMethods;
@@ -1450,7 +1454,7 @@ public class Compilation implements SourceLocator
 
     for (int j = 0;  j < numApplyMethods;  ++j)
       {
-	LambdaExp source = (LambdaExp) lexp.applyMethods.elementAt(j);
+	LambdaExp source = lexp.applyMethods.get(j);
         if (! source.usingCallContext())
           continue;
 	Method[] primMethods = source.primMethods;
@@ -1605,7 +1609,7 @@ public class Compilation implements SourceLocator
 
 	for (int j = 0;  j < numApplyMethods;  j++)
 	  {
-	    LambdaExp source = (LambdaExp) lexp.applyMethods.elementAt(j);
+	    LambdaExp source = lexp.applyMethods.get(j);
             if (source.usingCallContext())
               continue;
 	    // Select the subset of source.primMethods[*] that are suitable
@@ -1902,6 +1906,7 @@ public class Compilation implements SourceLocator
           {
             setState(BODY_PARSED-1);
             language.parse(this, 0);
+            mexp.classFor(this);
             lexer.close();
             lexer = null;
             setState(messages.seenErrors() ? ERROR_SEEN : BODY_PARSED);
@@ -1910,13 +1915,13 @@ public class Compilation implements SourceLocator
           }
         if (wantedState >= RESOLVED && getState() < RESOLVED)
           {
+            language.resolve(this);
             // Doing addMainClass is a bit flakey in the case that
             // ModuleExp.alwaysCompile is false.  We don't want to
             // call addMainClass *unless* we're compiling, but when
             // dealing with eval, mutually recursive modules, etc
             // it doesn't quite work.
             addMainClass(mexp);
-            language.resolve(this);
             setState(messages.seenErrors() ? ERROR_SEEN : RESOLVED);
           }
 
@@ -1994,7 +1999,8 @@ public class Compilation implements SourceLocator
       }
 
     ClassType neededSuper = getModuleType();
-    if (mainClass.getSuperclass().isSubtype(neededSuper))
+    if (mainClass.getSuperclass().isSubtype(neededSuper)
+        && ! module.getFlag(ModuleExp.USE_DEFINED_CLASS))
       moduleClass = mainClass;
     else
       {
@@ -2095,7 +2101,7 @@ public class Compilation implements SourceLocator
 	if (staticModule)
 	  {
             if (! module.getFlag(ModuleExp.USE_DEFINED_CLASS))
-              generateConstructor (module);
+              generateConstructor(module);
 
 	    code.emitNew(moduleClass);
 	    code.emitDup(moduleClass);
@@ -2449,6 +2455,18 @@ public class Compilation implements SourceLocator
     pushChain(scope, sc);
   }
 
+    public ScopeExp setPushCurrentScope (ScopeExp scope) {
+        ScopeExp old = currentScope();
+        lexical.pushSaveTopLevelRedefs();
+        setCurrentScope(scope);
+        return old;
+    }
+
+    public void setPopCurrentScope (ScopeExp old) {
+        setCurrentScope(old);
+        lexical.popSaveTopLevelRedefs();
+    }
+
   void pushChain (ScopeExp scope, ScopeExp limit)
   {
     if (scope != limit)
@@ -2681,10 +2699,10 @@ public class Compilation implements SourceLocator
   public LetExp letDone (Expression body)
   {
     LetExp let = (LetExp) current_scope;
-    if (let.getFlag(Expression.VALIDATED))
-      let.setFlag(false, Expression.VALIDATED);
-    else
+    // Check if letEnter has been called.
+    if (! let.getFlag(Expression.VALIDATED))
       letEnter();
+    let.setFlag(false, Expression.VALIDATED);
     let.body = body;
     pop(let);
     return let;
