@@ -335,7 +335,12 @@ public class Declaration
       {
         Object val;
         Expression value = getValueRaw();
-        if (field != null)
+        if (type == Type.javalangClassType
+            && (value instanceof ClassExp || value instanceof ModuleExp))
+          {
+            comp.loadClassRef(((LambdaExp) value).getCompiledClassType(comp));
+          }
+        else if (field != null)
           {
             comp.usedClass(field.getDeclaringClass());
             comp.usedClass(field.getType());
@@ -394,7 +399,6 @@ public class Declaration
         if (isIndirectBinding()
             && (flags & ReferenceExp.DONT_DEREFERENCE) == 0)
           {
-
             String filename;
             int line;
             if (access != null
@@ -502,7 +506,7 @@ public class Declaration
 
   /** True if in the function namespace, for languages that distinguishes them.
    * I.e. a function definition or macro definition. */
-  static final int PROCEDURE = 0x80;
+  public static final int PROCEDURE = 0x80;
 
   public static final int IS_ALIAS = 0x100;
 
@@ -536,7 +540,9 @@ public class Declaration
   /** If we need a 'context' supplied from a ReferenceExp or 'this. */
   public final boolean needsContext ()
   {
-    return base == null && field != null && ! field.getStaticFlag();
+    return base == null
+        && ((isClassField() && ! isStatic())
+            || (field != null && ! field.getStaticFlag()));
   }
 
   /** True if this is a field or method in a class definition. */
@@ -680,11 +686,14 @@ public class Declaration
 
   public void maybeIndirectBinding (Compilation comp)
   {
-    if (isLexical() 
-        && ! (context instanceof ModuleExp) || context == comp.mainLambda)
+      if (isLexical() && ! inExternalModule(comp))
       setIndirectBinding(true);
   }
 
+    public boolean inExternalModule(Compilation comp) {
+        return context instanceof ModuleExp && context != comp.mainLambda;
+    }
+
   /* Note:  You probably want to use !ignorable(). */
   public final boolean getCanRead() { return (flags & CAN_READ) != 0; }
   public final void setCanRead(boolean read)
@@ -779,7 +788,11 @@ public class Declaration
 
   /** List of ApplyExp where this declaration is the function called.
    * The applications are chained using their nextCall fields.
-   * The chain is not built if STATIC_SPECIFIED. */
+   * This is list is built twice:
+   * First in PushApply (for use during InlineCalls),
+   * then the list is clearest, then rebuilt in FindTailCalls.
+   * This is because InlineCalls may inline and re-arrange the call graph.
+   * The chain is not built the second time if STATIC_SPECIFIED. */
   public ApplyExp firstCall;
 
   /** Add a call to the list headed by {@code firstCall}. */
@@ -1091,7 +1104,8 @@ public class Declaration
     int fflags = 0;
     boolean isConstant = getFlag(IS_CONSTANT);
     boolean typeSpecified = getFlag(TYPE_SPECIFIED);
-    if (comp.immediate && context instanceof ModuleExp
+    if (comp.getModule().getFlag(ModuleExp.INTERACTIVE)
+        && context == comp.getModule()
         && ! isConstant && ! typeSpecified)
       setIndirectBinding(true);
     // In immediate mode we may need to access the field from a future
