@@ -338,7 +338,8 @@ public class InlineCalls extends ExpExpVisitor<Type> {
     if (decl != null && decl.field == null && ! decl.getCanWrite())
       {
         Expression dval = decl.getValue();
-        if (dval instanceof QuoteExp && dval != QuoteExp.undefined_exp)
+        if (dval instanceof QuoteExp && dval != QuoteExp.undefined_exp
+            && ! exp.getDontDereference())
           return visitQuoteExp(new QuoteExp(((QuoteExp) dval).getValue(), decl.getType()), required);
         // We don't want to visit the body of a named function yet.
         // Though not doing so does hurt optimization.
@@ -593,6 +594,10 @@ public class InlineCalls extends ExpExpVisitor<Type> {
       }
     if (exp.getClass() == LambdaExp.class)
       {
+          if (exp.canFinishCondition != LambdaExp.CAN_FINISH
+              && exp.canFinishCondition != null) {
+              exp.setReturnType(Type.neverReturnsType);
+          }
         Declaration ldecl = exp.nameDecl;
         boolean unknownCalls = true;
         if (ldecl != null && ! exp.isClassMethod() && ldecl.isModuleLocal())
