@@ -1,4 +1,4 @@
-// Copyright (c) 2002, 2003, 2004, 2005  Per M.A. Bothner.
+// Copyright (c) 2002, 2003, 2004, 2005, 2012  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
@@ -170,10 +170,9 @@ public abstract class Language
         || str.indexOf("-*- lisp -*-") >= 0)
       return getInstance("common-lisp");
     // Does it start with an XQuery comment or XQuery version statement?
-    if ((str.charAt(0) == '(' && str.charAt(1) == ':')
-        || (str.length() >= 7 && str.substring(0, 7).equals("xquery ")))
+    if (str.startsWith("(:") || str.startsWith("xquery"))
       return getInstance("xquery");
-    if (str.charAt(0) == ';' && str.charAt(1) == ';')
+    if (str.startsWith(";;"))
       return getInstance("scheme");
     return null;
   }
@@ -323,6 +322,24 @@ public abstract class Language
     environ.define(s, null, p);
   }
 
+  /**
+   * Get the corresponding {@link Type} for a given name.
+   * 
+   * This is currently used as a hook in the conversion of type designators to
+   * types. {@link gnu.kawa.lispexpr.LispLanguage} uses it to check for package
+   * style type designators such as {@code emacs:buffer},
+   * and CommonLisp uses it to check for a type designator "boolean",
+   * so that is can return the appropriate boolean type.
+   * This is a bit over-specialised, but it beats actually
+   * overriding getTypeFor(String) in LispLanguage, CommonLisp just for these minor
+   * changes... FIXME!
+   *
+   * @param name The name of a type to search for.
+   * @return The corresponding {@link Type} if a suitable one can be found,
+   *   otherwise {@code null}.
+   */
+  public Type getNamedType (String name) { return null; }
+
   /** Declare in the current Environment a variable aliased to a static field.
    */
   protected void defAliasStFld(String name, String cname, String fname)
@@ -674,15 +691,20 @@ public abstract class Language
       tr.explicit = true;
     if ((options & PARSE_PROLOG) != 0)
       tr.setState(Compilation.PROLOG_PARSING);
-    tr.pushNewModule(lexer);
+    ModuleExp module = tr.pushNewModule(lexer);
     if ((options & PARSE_INTERACTIVE_MODULE) != 0)
-      tr.setInteractiveName();
+      {
+        tr.setInteractiveName();
+        module.setFlag(ModuleExp.INTERACTIVE);
+      }
     if (info != null)
       info.setCompilation(tr);
     if (! parse(tr, options))
       return null;
     if (tr.getState() == Compilation.PROLOG_PARSING)
       tr.setState(Compilation.PROLOG_PARSED);
+    else
+      module.classFor(tr);
     return tr;
   }
 
@@ -696,25 +718,26 @@ public abstract class Language
   {
   }
 
-  public Type getTypeFor(Class clas)
-  {
-    return Type.make(clas);
-  }
+    // FIXME: Would be better and little fuss to use a perfect hash
+    // function for this.
+    public Type getTypeFor(Class clas) {
+        return Type.make(clas);
+    }
 
     public final Type getLangTypeFor (Type type) {
 	if (type instanceof ParameterizedType) {
-	    ParameterizedType ptype = (ParameterizedType) type;
-	    Type[] pargs = ptype.getTypeArgumentTypes();
-	    if (ptype.getRawType() == LazyType.lazyType && pargs.length == 1) {
+            ParameterizedType ptype = (ParameterizedType) type;
+            Type[] pargs = ptype.getTypeArgumentTypes();
+            if (ptype.getRawType() == LazyType.lazyType && pargs.length == 1) {
 		return LazyType.getInstance(LazyType.lazyType, getLangTypeFor(pargs[0]));
-	    }
+            }
 	}
 	if (type instanceof TypeVariable) {
-	    return getLangTypeFor(((TypeVariable) type).getRawType());
+            return getLangTypeFor(((TypeVariable) type).getRawType());
 	}
 	if (type.isExisting()) {
-	    Class clas = type.getReflectClass();
-	    if (clas != null)
+            Class clas = type.getReflectClass();
+            if (clas != null)
 		return getTypeFor(clas);
 	}
 	return type;
@@ -728,27 +751,23 @@ public abstract class Language
     return s;
   }
 
-  public static Type string2Type (String name)
-  {
-    Type t;
-    if (name.endsWith("[]"))
-      {
-	t = string2Type(name.substring(0, name.length()-2));
-	if (t == null)
-	  return null;
-	t = gnu.bytecode.ArrayType.make(t);
-      }
-    else if (gnu.bytecode.Type.isValidJavaTypeName(name))
-      t = gnu.bytecode.Type.getType(name);
-    else
-      return null;
-    return t;
-  }
-
-  public Type getTypeFor (String name)
-  {
-    return string2Type(name);
-  }
+    public Type getTypeFor (String name) {
+        Type t;
+        if (name.endsWith("[]")) {
+            t = getTypeFor(name.substring(0, name.length() - 2));
+            if (t != null)
+                t = ArrayType.make(t);
+        }
+        else {
+            t = getNamedType(name);
+        }
+        if (t != null)
+            return t;
+        else if (Type.isValidJavaTypeName(name)) {
+            t = Type.getType(name);
+        }
+        return t;
+    }
 
   public final Type getTypeFor (Object spec, boolean lenient)
   {
@@ -766,7 +785,7 @@ public abstract class Language
       {
         String uri = ((Namespace) spec).getName();
         if (uri != null && uri.startsWith("class:"))
-          return getLangTypeFor(string2Type(uri.substring(6)));
+          return getLangTypeFor(getTypeFor(uri.substring(6)));
       }
     return null;
   }
@@ -897,7 +916,8 @@ public abstract class Language
     boolean isFinal = (fld.getModifiers() & Access.FINAL) != 0;
     if ((isImportedInstance = fname.endsWith("$instance")))
       fdname = fname;
-    else if (isFinal && fvalue instanceof Named) // && ! isAlias
+    else if (isFinal && ftype == Compilation.typeModuleMethod
+             && fvalue instanceof Named /* should always be true */)
       fdname = ((Named) fvalue).getSymbol();
     else
       {
