@@ -195,6 +195,10 @@ public class ReferenceExp extends AccessExp
             Expression dval = decl.getValue();
             if (dval != null)
               return dval.validateApply(exp, visitor, required, decl);
+            Type dtype = decl.type;
+            if (dtype instanceof ClassType
+                && ((ClassType) dtype).isSubclass("kawa.lang.Continuation"))
+                exp.setType(Type.neverReturnsType);
           }
       }
     else if (getSymbol() instanceof Symbol)
@@ -212,6 +216,8 @@ public class ReferenceExp extends AccessExp
   {
     ps.print("(Ref/");
     ps.print(id);
+    if (getDontDereference())
+      ps.print(",dont-deref");
     if (symbol != null
 	&& (binding == null || symbol.toString() != binding.getName()))
       {
@@ -240,7 +246,9 @@ public class ReferenceExp extends AccessExp
         return Compilation.typeLocation;
       }
     decl = Declaration.followAliases(decl);
-    Type type = decl.getType();
+    Type type = decl.isAlias() && decl.isIndirectBinding()
+        ? Type.objectType // FIXME - should use a parameterized Location type
+        : decl.getType();
     if (type == Type.toStringType)
       type = Type.javalangStringType;
     return type;
