@@ -129,44 +129,39 @@ public class ApplyToArgs extends ProcedureN
 
   Language language;
 
-  public Object applyN (Object[] args) throws Throwable
-  {
-    Object proc = Promise.force(args[0]);
-    if (proc instanceof Procedure)
-      {
-        Object[] rargs = new Object[args.length-1];
-        System.arraycopy(args, 1, rargs, 0, rargs.length);
-        return ((Procedure) proc).applyN(rargs);
-      }
-    if (proc instanceof gnu.bytecode.Type
-        || proc instanceof Class)
-      {
-        return gnu.kawa.reflect.Invoke.make.applyN(args);
-      }
-    if (proc instanceof
-        /* #ifdef JAVA2 */
-        java.util.List
-        /* #else */
-        // gnu.lists.Sequence
-        /* #endif */
-        )
-      {
-        if (args.length != 2)
-          throw new WrongArguments(this, args.length); // FIXME
-        int index = ((Number) Promise.force(args[1])).intValue();
-        /* #ifdef JAVA2 */
-        return ((java.util.List) proc).get(index);
-        /* #else */
-        // return ((gnu.lists.Sequence) proc).get(index);
-        /* #endif */
-      }
-    Class pclass = proc.getClass();
-    if (pclass.isArray())
-      {
-        if (args.length != 2)
-          throw new WrongArguments(this, args.length); // FIXME
-        return java.lang.reflect.Array.get(proc, ((Number) args[1]).intValue());
-      }
-    throw new WrongType(this, 0, proc, "procedure");
-  }
+    public Object applyN (Object[] args) throws Throwable {
+        Object proc = Promise.force(args[0]);
+        if (proc instanceof Procedure) {
+            Object[] rargs = new Object[args.length-1];
+            System.arraycopy(args, 1, rargs, 0, rargs.length);
+            return ((Procedure) proc).applyN(rargs);
+        }
+        if (proc instanceof gnu.bytecode.Type
+            || proc instanceof Class) {
+            return gnu.kawa.reflect.Invoke.make.applyN(args);
+        }
+        if (proc instanceof java.util.List) {
+            if (args.length != 2)
+                throw new WrongArguments(this, args.length); // FIXME
+            int index = ((Number) Promise.force(args[1])).intValue();
+            return ((java.util.List) proc).get(index);
+        }
+        /*
+          What should happen if key has no associated value?
+          Throw an exception?  Return null?
+        if (proc instanceof java.util.Map) {
+            if (args.length != 2)
+                throw new WrongArguments(this, args.length); // FIXME
+            Object key = Promise.force(args[1]);
+            
+        }
+        */
+        Class pclass = proc.getClass();
+        if (pclass.isArray()) {
+            if (args.length != 2)
+                throw new WrongArguments(this, args.length); // FIXME
+            return java.lang.reflect.Array.get(proc, ((Number) args[1]).intValue());
+        }
+        throw new WrongType(this, 0, proc, "procedure");
+    }
 }
