@@ -680,9 +680,14 @@ public class CompileMisc implements Inlineable
     Compilation comp = visitor.getCompilation();
 
     // First an outer (let ((%proc PROC)) L2), where PROC is args[0].
-    comp.letStart();
-    Declaration procDecl = comp.letVariable("%proc", Compilation.typeProcedure,
-                                            proc);
+    if (! procSafeForMultipleEvaluation)
+      {
+        comp.letStart();
+        Declaration procDecl = comp.letVariable("%proc",
+                                                Compilation.typeProcedure,
+                                                proc);
+        proc = new ReferenceExp(procDecl);
+      }
 
     // Then an inner L2=(let ((%loop (lambda (argi ...) ...))) (%loop ...))
     comp.letStart();
@@ -706,14 +711,12 @@ public class CompileMisc implements Inlineable
     Expression[] recArgs = new Expression[collect ? nargs + 1 : nargs];
     for (int i = 0;  i < nargs;  i++)
       {
-	doArgs[i+1] = visitor.visitApplyOnly(SlotGet.makeGetField(new ReferenceExp(pargs[i]), "car"), null);
-	recArgs[i] = visitor.visitApplyOnly(SlotGet.makeGetField(new ReferenceExp(pargs[i]), "cdr"), null);
+        doArgs[i+1] = SlotGet.makeGetField(new ReferenceExp(pargs[i]), "car");
+	recArgs[i] = SlotGet.makeGetField(new ReferenceExp(pargs[i]), "cdr");
       }
-    if (! procSafeForMultipleEvaluation)
-      proc = new ReferenceExp(procDecl);
     doArgs[0] = proc;
     Expression applyFunc = new ReferenceExp(SchemeCompilation.applyFieldDecl);
-    Expression doit = visitor.visitApplyOnly(new ApplyExp(applyFunc, doArgs), null);
+    Expression doit = new ApplyExp(applyFunc, doArgs);
     if (collect)
       {
 	Expression[] consArgs = new Expression[2];
@@ -722,7 +725,7 @@ public class CompileMisc implements Inlineable
 	recArgs[nargs] = Invoke.makeInvokeStatic(Compilation.typePair,
 						 "make", consArgs);
       }
-    Expression rec = visitor.visitApplyOnly(new ApplyExp(new ReferenceExp(loopDecl), recArgs), null);
+    Expression rec = new ApplyExp(new ReferenceExp(loopDecl), recArgs);
     lexp.body = comp.letDone(collect ? rec : new BeginExp(doit, rec));
     Expression[] initArgs = new Expression[collect ? nargs + 1 : nargs];
     QuoteExp empty = new QuoteExp(LList.Empty);
@@ -734,14 +737,14 @@ public class CompileMisc implements Inlineable
 	Expression result
 	  = collect ? (Expression) new ReferenceExp(resultDecl)
 	  : (Expression) QuoteExp.voidExp;
-	lexp.body = new IfExp(visitor.visitApplyOnly(new ApplyExp(mproc.isEq, compArgs), null),
+	lexp.body = new IfExp(new ApplyExp(mproc.isEq, compArgs),
 			      result, lexp.body);
 	initArgs[i] = args[i+1];
       }
     if (collect)
       initArgs[nargs] = empty;
 
-    Expression body = visitor.visitApplyOnly(new ApplyExp(new ReferenceExp(loopDecl), initArgs), null);
+    Expression body = new ApplyExp(new ReferenceExp(loopDecl), initArgs);
     if (collect)
       {
 	Expression[] reverseArgs = new Expression[1];
@@ -749,13 +752,10 @@ public class CompileMisc implements Inlineable
 	body = Invoke.makeInvokeStatic(Compilation.scmListType,
 				       "reverseInPlace", reverseArgs);
       }
-    LetExp let2 = comp.letDone(body);
-    LetExp let1 = comp.letDone(let2);
-
-    if (procSafeForMultipleEvaluation)
-      return let2;
-    else
-      return let1;
+    LetExp ret = comp.letDone(body);
+    if (! procSafeForMultipleEvaluation)
+      ret = comp.letDone(ret);
+    return visitor.visit(ret, null);
   }
 
   public static Expression validateApplyMakePromise
