@@ -3,6 +3,7 @@ import gnu.math.*;
 import gnu.mapping.*;
 import gnu.text.Char;
 import gnu.expr.Language;
+import gnu.mapping.Promise;
 
 /** Implement that standard Scheme function "eqv?". */
 
@@ -22,6 +23,8 @@ public class IsEqv extends Procedure2
 
   public static boolean apply (Object arg1, Object arg2) 
   {
+    arg1 = Promise.force(arg1);
+    arg2 = Promise.force(arg2);
     if (arg1==arg2)
       return true;
     if (arg1 instanceof Number && arg2 instanceof Number)
