@@ -7,6 +7,7 @@ import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.xml.*;
 import gnu.lists.*;
+import java.util.List;
 
 public abstract class NodeConstructor extends MethodProc
 implements Inlineable
@@ -14,6 +15,15 @@ implements Inlineable
   public abstract void compileToNode (ApplyExp exp, Compilation comp,
 				      ConsumerTarget target);
 
+    /** If true, top-level strings are treated as text nodes.
+     * This means don't separate them with spaces when printing as XML.
+     */
+    public void setStringIsText(boolean stringIsText) {
+        this.stringIsText = stringIsText;
+    }
+
+    protected boolean stringIsText;
+
   public static XMLFilter pushNodeConsumer (Consumer out)
   {
     if (out instanceof XMLFilter)
@@ -59,7 +69,7 @@ implements Inlineable
       }
   }
 
-  public static void compileChild (Expression arg,
+    public static void compileChild (Expression arg, boolean stringIsText,
 				   Compilation comp, ConsumerTarget target)
   {
     if (arg instanceof ApplyExp)
@@ -76,7 +86,11 @@ implements Inlineable
 	      }
 	  }
       }
-    arg.compileWithPosition(comp, target);
+    CodeAttr code = comp.getCode();
+    arg.compileWithPosition(comp, Target.pushObject);
+    code.emitLoad(target.getConsumerVariable());
+    code.emitInvokeStatic(ClassType.make("gnu.kawa.xml.NodeConstructor")
+                          .getDeclaredMethod(stringIsText ? "writeContentS" : "writeContent", 2));
   }
 
   /** Compile an expression using a fresh NodeTree.
@@ -161,6 +175,31 @@ implements Inlineable
     return Compilation.typeObject;
   }
 
+    public static void writeContentS(Object arg, Consumer out) {
+          if (arg instanceof CharSequence && ! (arg instanceof UnescapedData)) {
+                CharSequence carg = (CharSequence) arg;
+                out.write(carg, 0, carg.length());
+            }
+          else
+              writeContent(arg, out);
+    }
+    public static void writeContent(Object arg, Consumer out) {
+        if (arg instanceof List  && ! (arg instanceof CharSequence)) {
+            for (Object e : (List) arg) {
+                writeContent1(e, out);
+            }
+        }
+        else
+            writeContent1(arg, out);
+    }
+
+    protected static void writeContent1(Object arg, Consumer out) {
+        if (arg instanceof Consumable)
+            ((Consumable) arg).consume(out);
+        else
+            Values.writeValues(arg, out);
+    }
+
   static final ClassType typeXMLFilter
     = ClassType.make("gnu.xml.XMLFilter");
   static final ClassType typeKNode
