@@ -3,6 +3,7 @@
 
 package gnu.lists;
 import java.io.*;
+import java.util.Arrays;
 
 /** Simple adjustable-length vector whose elements are Object references.
  */
@@ -67,6 +68,13 @@ public class FVector<E> extends SimpleVector<E>
     return new FVector(data);
   }
 
+    public void copyFrom (int index, FVector src, int start, int end) {
+        int count = end-start;
+        if (count < 0 || index+count > size || end > src.size)
+            throw new ArrayIndexOutOfBoundsException();
+        System.arraycopy(src.data, start, data, index, count);
+    }
+
   /** Get the allocated length of the data buffer. */
   public int getBufferLength()
   {
@@ -170,14 +178,10 @@ public class FVector<E> extends SimpleVector<E>
   }
   */
 
-  // FIXME - bad name - setAll should take a Collection
-  public final void setAll (Object new_value)
-  {
-    checkCanWrite();
-    Object[] d = data;
-    for (int i = size; --i >= 0; )
-      d[i] = new_value;
-  }
+    public final void fill(Object new_value, int start, int end) {
+        checkCanWrite();
+        Arrays.fill(data, start, end, new_value);
+    }
 
   public boolean consumeNext(int ipos, Consumer out)
   {
