@@ -12,20 +12,19 @@ public class FilePath
   extends Path
   /* #ifdef JAVA2 */
   /* #ifdef JAVA5 */
-  // implements Comparable<FilePath>
+  implements Comparable<FilePath>
   /* #else */
-  implements Comparable
+  // implements Comparable
   /* #endif */
   /* #endif */
 {
-  File file;
+  final File file;
   /** Usually the same as {@code file.toString()}.
    * One important difference: {@code isDirectory} is true
    * if {@code path} ends with the {@code '/'} or the {@code separatorChar}.
-   * (
    * The original String if constructed from a String.
    */
-  String path;
+  final String path;
 
   private FilePath (File file)
   {
@@ -124,6 +123,11 @@ public class FilePath
     return false;
   }
 
+  public boolean delete ()
+  {
+    return toFile().delete();
+  }
+
   public long getLastModified ()
   {
     return file.lastModified();
@@ -152,9 +156,9 @@ public class FilePath
 
   public
   /* #ifdef JAVA5 */
-  // FilePath
+  FilePath
   /* #else */
-  Path
+  // Path
   /* #endif */
   getParent ()
   {
@@ -171,10 +175,10 @@ public class FilePath
   }
 
   /* #ifndef JAVA5 */
-  public int compareTo (Object obj)
-  {
-    return compareTo((FilePath) obj);
-  }
+  // public int compareTo (Object obj)
+  // {
+  //   return compareTo((FilePath) obj);
+  // }
   /* #endif */
 
   public boolean equals (Object obj)
@@ -233,7 +237,7 @@ public class FilePath
   /* #endif */
 
   /* #ifdef use:java.net.URI */
-  private static URI toURI (File file)
+  private static URI toUri (File file)
   {
     try
       {
@@ -254,11 +258,11 @@ public class FilePath
 
   }
 
-  public URI toURI ()
+  public URI toUri ()
   {
     if (this == Path.userDirPath)
       return resolve("").toURI();
-    return toURI(file);
+    return toUri(file);
   }
   /* #endif */
 
