@@ -444,6 +444,8 @@ public class XMLFilter implements
                 else
                   uri = resolve(prefix, i > 0);
               }
+            else if (! (saved instanceof Symbol))
+                throw new ClassCastException("expected element start tag (a symbol) - instead got a "+saved.getClass().getName());
             else
               {
                 Symbol symbol = (Symbol) saved;
@@ -696,7 +698,9 @@ public class XMLFilter implements
       }
     else
       {
-        closeStartTag();
+        boolean inImplicitAttr = previous == SAW_KEYWORD;
+        if (! inImplicitAttr)
+          closeStartTag();
         if (v instanceof UnescapedData)
           {
             base.writeObject(v);
@@ -709,6 +713,8 @@ public class XMLFilter implements
             TextUtils.textValue(v, this);  // Atomize.
             previous = SAW_WORD;
           }
+        if (inImplicitAttr)
+          endAttribute();
       }
   }
 
