@@ -58,8 +58,8 @@ public class XMLPrinter extends OutPort
    * If non-null, this is the the system identifier. */
   public static final ThreadLocation doctypePublic
     = new ThreadLocation("doctype-public");
-  public static final ThreadLocation indentLoc
-    = new ThreadLocation("xml-indent");
+  public static final ThreadLocation<String> indentLoc
+    = new ThreadLocation<String>("xml-indent");
 
   public boolean strict;
 
@@ -740,7 +740,7 @@ public class XMLPrinter extends OutPort
           prev = '-';
 	return;
       }
-    if (v instanceof Consumable && ! (v instanceof UnescapedData))
+    if (v instanceof Consumable)
       {
 	((Consumable) v).consume(this);
 	return;
