@@ -97,11 +97,13 @@ public class CompileMisc
 
     exp.setType(args[0].getType());
 
-    Compilation parser = visitor.getCompilation();
+    Compilation comp = visitor.getCompilation();
 
     Declaration dotArg = lexp2.firstDecl();
     Declaration posArg = dotArg.nextDecl();
     Declaration lastArg = posArg.nextDecl();
+    dotArg.setCanRead(true);
+    posArg.setCanRead(true);
 
     lexp2.setInlineOnly(true);
     lexp2.returnContinuation = exp;
@@ -117,8 +119,9 @@ public class CompileMisc
         // Don't need to do anything more - lastArg is not needed.
         return exp;
       }
+    lastArg.setCanRead(true);
 
-    parser.letStart();
+    comp.letStart();
     Expression seq = args[0];
     Type seqType;
     Method sizeMethod;
@@ -133,8 +136,8 @@ public class CompileMisc
         seq = new ApplyExp(SortNodes.sortNodes, new Expression [] {seq});
         sizeMethod = CoerceNodes.typeNodes.getDeclaredMethod("size", 0);
       }
-    Declaration sequence = parser.letVariable("sequence", seqType, seq);
-    parser.letEnter();
+    Declaration sequence = comp.letVariable("sequence", seqType, seq);
+    comp.letEnter();
 
     Expression pred = lexp2.body;
     Type predType = lexp2.body.getType();
@@ -145,6 +148,7 @@ public class CompileMisc
     if (vproc.kind == 'R')
       {
         Declaration posIncoming = new Declaration(null, Type.intType);
+        posIncoming.setCanRead(true);
 	Expression init
 	  = new ApplyExp(AddOp.$Mn,
 			 new Expression[] {
@@ -155,11 +159,12 @@ public class CompileMisc
 			 new Expression[] {
 			   init,
 			   new QuoteExp(IntNum.one())});
-        LetExp let = new LetExp(new Expression[] { init });
+        
+        comp.letStart();
         lexp2.replaceFollowing(dotArg, posIncoming);
-        let.add(posArg);
-        let.body = pred;
-        pred = let;
+        comp.letVariable(posArg, init);
+        comp.letEnter();
+        pred = comp.letDone(pred);
       }
 
     pred = new IfExp(pred,
@@ -178,11 +183,11 @@ public class CompileMisc
                                        new Expression[] {
                                          new ReferenceExp(sequence)});
 
-    LetExp let2 = new LetExp(new Expression[] { lastInit });
-    let2.add(lastArg);
-    let2.body = gnu.kawa.functions.CompileMisc.validateApplyValuesMap(doMap, visitor, required, ValuesMap.valuesMapWithPos);
+    comp.letStart();
+    comp.letVariable(lastArg, lastInit);
+    LetExp let2 = comp.letDone(gnu.kawa.functions.CompileMisc.validateApplyValuesMap(doMap, visitor, required, ValuesMap.valuesMapWithPos));
 
-    return parser.letDone(let2);
+    return comp.letDone(let2);
   }
 
   public static Expression validateApplyRelativeStep
@@ -254,13 +259,15 @@ public class CompileMisc
 
         Method sizeMethod = typeNodes.getDeclaredMethod("size", 0);
         Expression lastInit
-          =  new ApplyExp(sizeMethod,
-                          new Expression[] {new ReferenceExp(sequence)});
-        LetExp lastLet = new LetExp(new Expression[] { lastInit });
-        lastLet.addDeclaration(lastArg);
-        lastLet.body = new ApplyExp(exp.getFunction(),
+          = new ApplyExp(sizeMethod,
+                         new Expression[] {new ReferenceExp(sequence)});
+        comp.letStart();
+        comp.letVariable(lastArg, lastInit);
+        comp.letEnter();
+        LetExp lastLet =
+          comp.letDone(new ApplyExp(exp.getFunction(),
                                     new Expression[] { new ReferenceExp(sequence),
-                                                       lexp2 });
+                                                       lexp2 }));
         return comp.letDone(lastLet);
       }
 
