@@ -14,7 +14,7 @@ import java.util.regex.Matcher;
 /* #endif */
 import gnu.text.*;
 /* #ifdef use:java.text.Normalizer */
-// import java.text.Normalizer;
+import java.text.Normalizer;
 /* #endif */
 
 public class StringUtils
@@ -269,18 +269,19 @@ public class StringUtils
     StringBuffer s = new StringBuffer();
     String glue = coerceToString(join, "string-join", 2, ERROR_VALUE);
     int glen = glue.length();
-    int index=0;
+    int index=0, next;
     boolean started = false;
 
-    while((index=Values.nextIndex(strseq, index)) >= 0)
+    while((next=Values.nextIndex(strseq, index)) >= 0)
       {
-	Object obj = Values.nextValue(strseq, index-1);
+	Object obj = Values.nextValue(strseq, index);
 	if (obj == Values.empty) continue;
 
 	if (started && glen > 0)
           s.append(glue);
         s.append(TextUtils.stringValue(obj));
 	started=true;
+        index = next;
       }
 
     return s.toString();
@@ -592,20 +593,31 @@ public class StringUtils
     if ("".equals(form))
       return str;
     /* #ifdef use:java.text.Normalizer */
-    // Normalizer.Form nform;
-    // if ("NFC".equals(form))
-    //   nform = Normalizer.Form.NFC;
-    // else if ("NFD".equals(form))
-    //   nform = Normalizer.Form.NFD;
-    // else if ("NFKC".equals(form))
-    //   nform = Normalizer.Form.NFKC;
-    // else if ("NFKD".equals(form))
-    //   nform = Normalizer.Form.NFKD;
-    // else
-    //   throw new RuntimeException("normalize-unicode: unknown normalization form '"+form+'\'');
-    // return Normalizer.normalize(str, nform);
+    /* #ifdef JAVA6COMPAT5 */
+    // try
+    //   {
+    /* #endif */
+        Normalizer.Form nform;
+        if ("NFC".equals(form))
+          nform = Normalizer.Form.NFC;
+        else if ("NFD".equals(form))
+          nform = Normalizer.Form.NFD;
+        else if ("NFKC".equals(form))
+          nform = Normalizer.Form.NFKC;
+        else if ("NFKD".equals(form))
+          nform = Normalizer.Form.NFKD;
+        else
+          throw new RuntimeException("normalize-unicode: unknown normalization form '"+form+'\'');
+        return Normalizer.normalize(str, nform);
+    /* #ifdef JAVA6COMPAT5 */
+    //   }
+    // catch (java.lang.NoClassDefFoundError ex)
+    //   {
+    //     throw new UnsupportedOperationException("normalize-unicode: unicode string normalization not available");
+    //   }
+    /* #endif */
     /* #else */
-    throw AbstractSequence.unsupportedException("normalize-unicode form "+form);
+    // throw new UnsupportedOperationException("normalize-unicode: unicode string normalization not available");
     /* #endif */
   }
 }
