@@ -1,6 +1,5 @@
 package kawa.lang;
-import gnu.bytecode.Type;
-import gnu.bytecode.ClassType;
+import gnu.bytecode.*;
 import gnu.mapping.*;
 import gnu.expr.*;
 
@@ -9,26 +8,22 @@ import gnu.expr.*;
 public class GetFieldProc extends Procedure1 implements Inlineable
 {
   ClassType ctype;
-  String fname;
-  gnu.bytecode.Field field;
-  java.lang.reflect.Field reflectField;
+  Field field;
 
-  GetFieldProc (Class clas, String fname)
+  public GetFieldProc (Class clas, String fname)
   {
-    ctype = (ClassType) gnu.bytecode.Type.make(clas);
-    this.fname = fname;
+    this ((ClassType) Type.make(clas), fname);
   }
 
   public GetFieldProc (ClassType ctype, String fname)
   {
     this.ctype = ctype;
-    this.fname = fname;
+    this.field = Field.searchField(ctype.getFields(), fname);
   }
 
   public GetFieldProc (ClassType ctype, String name, Type ftype, int flags)
   {
     this.ctype = ctype;
-    this.fname = name;
     field = ctype.getField(name);
     if (field == null)
       field = ctype.addField(name, ftype, flags);
@@ -36,37 +31,25 @@ public class GetFieldProc extends Procedure1 implements Inlineable
 
   public Object apply1 (Object arg1)
   {
-    if (reflectField == null)
-      {
-	Class clas = ctype.getReflectClass();
-	try
-	  {
-	    reflectField = clas.getField (fname);
-	  }
-	catch (NoSuchFieldException ex)
-	  {
-	    throw new RuntimeException ("no such field "+fname+" in "+clas.getName());
-	  }
-      }
     try
       {
+	java.lang.reflect.Field reflectField = field.getReflectField();
 	return reflectField.get(arg1);
       }
+    catch (NoSuchFieldException ex)
+      {
+	throw new RuntimeException ("no such field " + field.getSourceName()
+				    + " in " + ctype.getName());
+      }
     catch (IllegalAccessException ex)
       {
-	throw new RuntimeException("illegal access for field "+fname);
+	throw new RuntimeException("illegal access for field "
+				   + field.getSourceName());
       }
   }
 
   private gnu.bytecode.Field getField ()
   {
-    if (field == null)
-      {
-	field = ctype.getField(fname);
-	if (field == null)
-	  field = ctype.addField(fname, Type.make(reflectField.getType()),
-				 reflectField.getModifiers());
-      }
     return field;
   }
 
@@ -79,7 +62,6 @@ public class GetFieldProc extends Procedure1 implements Inlineable
         return;
       }
     exp.getArgs()[0].compile(comp, ctype);
-    getField();
     gnu.bytecode.CodeAttr code = comp.getCode();
     code.emitGetField(field);
     target.compileFromStack(comp, field.getType());
