@@ -16,16 +16,17 @@ public class TemplateScope extends LetExp implements Externalizable
    * This is used to setContextDecl() of such references. */
   Declaration macroContext;
 
+  /** See Translator#currentMacroMark. */
+  Object macroMark;
+
   private Syntax syntax; // Only used for debugging
 
   public TemplateScope ()
   {
-    super(null);
   }
 
   public TemplateScope (ScopeExp outer)
   {
-    super(null);
     this.outer = outer;
   }
 
@@ -34,18 +35,19 @@ public class TemplateScope extends LetExp implements Externalizable
     return make((Translator) Compilation.getCurrent());
   }
 
-  public static TemplateScope make (Translator tr)
-  {
-    TemplateScope templateScope = new TemplateScope();
-    Syntax curSyntax = tr.getCurrentSyntax();
-    if (curSyntax instanceof Macro)
-      {
-        templateScope.outer = ((Macro) curSyntax).getCapturedScope();
-        templateScope.macroContext = tr.macroContext;
-      }
-    templateScope.syntax = curSyntax;
-    return templateScope;
-  }
+    public static TemplateScope make (Translator tr) {
+        TemplateScope templateScope = new TemplateScope();
+        if (tr != null) {
+            templateScope.macroMark = tr.currentMacroMark;
+            Syntax curSyntax = tr.getCurrentSyntax();
+            if (curSyntax instanceof Macro) {
+                templateScope.outer = ((Macro) curSyntax).getCapturedScope();
+                templateScope.macroContext = tr.macroContext;
+            }
+            templateScope.syntax = curSyntax;
+        }
+        return templateScope;
+    }
 
   public String toString() { return super.toString()+"(for "+syntax+")"; }
 
