@@ -2,12 +2,13 @@
 
 ;;; Definitions for some standard syntax.
 
-(module-export cond case and or let let* do delay
-	       syntax-object->datum datum->syntax-object with-syntax
+(module-export cond case and or let let* do delay lazy
+               syntax->datum datum->syntax with-syntax
+	       syntax-object->datum datum->syntax-object ; deprecated
 	       begin-for-syntax define-for-syntax
 	       generate-temporaries define-procedure
-	       identifier? free-identifier=?
-	       syntax-source syntax-line syntax-column)
+	       identifier? free-identifier=? bound-identifier=?
+	       syntax-source syntax-line syntax-column eval)
 
 ;;; COND
 
@@ -234,9 +235,20 @@
 
 ;;; DELAY
 
-(define-syntax delay (syntax-rules ()
-				   ((delay expression)
-				    (make <kawa.lang.Promise> (lambda () expression)))))
+;; See racket-5.2/collects/racket/private/promise.rkt
+;; See SRFI-45
+
+(define-syntax (lazy form)
+  (syntax-case form ()
+    ((lazy expression)
+     (gnu.expr.ApplyExp gnu.kawa.functions.MakePromise:makeLazy
+                        (syntax->expression #'(lambda () expression))))))
+
+(define-syntax (delay form)
+  (syntax-case form ()
+    ((delay expression)
+     (gnu.expr.ApplyExp gnu.kawa.functions.MakePromise:makeDelay
+                        (syntax->expression #'(lambda () expression))))))
 
 (define-syntax define-procedure
   (syntax-rules (:: <gnu.expr.GenericProc>)
@@ -249,12 +261,18 @@
 		     (make <gnu.expr.GenericProc> 'name))
 		   (invoke name 'setProperties (java.lang.Object[] args ...))))))
 
-(define (syntax-object->datum obj)
+(define (syntax->datum obj)
   (kawa.lang.Quote:quote obj))
 
-(define (datum->syntax-object template-identifier obj)
+(define (syntax-object->datum obj)
+  (syntax->datum obj))
+
+(define (datum->syntax template-identifier obj)
   (kawa.lang.SyntaxForms:makeWithTemplate template-identifier obj))
 
+(define (datum->syntax-object template-identifier obj)
+  (datum->syntax template-identifier obj))
+
 (define (generate-temporaries list)
   (let loop ((n (kawa.lang.Translator:listLength list)) (lst '()))
     (if (= n 0) lst
@@ -266,7 +284,14 @@
 	   (kawa.lang.SyntaxForms:isIdentifier form))))
 
 (define (free-identifier=? id1 id2) :: <boolean>
-  (kawa.lang.SyntaxForms:freeIdentifierEquals id1 id2))
+  (if (and (identifier? id1) (identifier? id2))
+      (kawa.lang.SyntaxForms:identifierEquals id1 id2 #f)
+      (syntax-error "free-identifier-? - argument is not an identifier")))
+
+(define (bound-identifier=? id1 id2) :: <boolean>
+  (if (and (identifier? id1) (identifier? id2))
+      (kawa.lang.SyntaxForms:identifierEquals id1 id2 #t)
+      (syntax-error "bound-identifier-? - argument is not an identifier")))
 
 (define (syntax-source form)
   (cond ((instance? form kawa.lang.SyntaxForm)
@@ -294,6 +319,11 @@
 	(else
 	 #f)))
 
+(with-compile-options
+ full-tailcalls: #t
+ (define (eval exp #!optional (env ::gnu.mapping.Environment (gnu.mapping.Environment:user)))
+   (kawa.lang.Eval:evalForm exp env)))
+
 (define-syntax begin-for-syntax
   (lambda (form)
     (syntax-case form ()
