@@ -20,7 +20,7 @@
      ((define-syntax-case name literals . parts)
       (define-syntax name
         (lambda (form)
-      (syntax-case form literals . parts))))))
+          (syntax-case form literals . parts))))))
 
 (define-syntax when (syntax-rules ()
 				  ((when cond exp ...)
@@ -30,19 +30,21 @@
 				  ((when cond exp ...)
 				   (if (not cond) (begin exp ...)))))
 
-(define-syntax (try-finally x)
-  (syntax-case x ()
-	       ((_ try-part finally-part)
-		(make <gnu.expr.TryExp>
-		  (syntax->expression (syntax try-part))
-		  (syntax->expression (syntax finally-part))))))
+(define-rewrite-syntax try-finally
+  (lambda (x)
+    (syntax-case x ()
+      ((_ try-part finally-part)
+       (make <gnu.expr.TryExp>
+         (syntax->expression (syntax try-part))
+         (syntax->expression (syntax finally-part)))))))
 
-(define-syntax (synchronized x)
-  (syntax-case x ()
-	       ((_ object . body)
-		(make <gnu.expr.SynchronizedExp>
-		  (syntax->expression (syntax object))
-		  (syntax-body->expression (syntax body))))))
+(define-rewrite-syntax synchronized
+  (lambda (x)
+    (syntax-case x ()
+      ((_ object . body)
+       (make <gnu.expr.SynchronizedExp>
+         (syntax->expression (syntax object))
+         (syntax-body->expression (syntax body)))))))
 
 (define (identifier-list? obj) ::boolean
   (and (>= (kawa.lang.Translator:listLength obj) 0)
@@ -94,24 +96,24 @@
 (define-syntax-case %import (library only except prefix rename)
   ((_ (rename import-set . pairs) mapper)
    (if (identifier-pair-list? #'pairs)
-       #`(%import import-set ,(cons (cons import-handle-rename #`pairs) #`mapper))
+       #`(%import import-set #,(cons (cons import-handle-rename #`pairs) #`mapper))
        (syntax-error (syntax rest) "invalid 'rename' clause in import")))
   ((_ (only import-set . ids) mapper)
    (if (identifier-list? #'ids)
-       #`(%import import-set ,(cons (cons import-handle-only #`ids) #`mapper))
+       #`(%import import-set #,(cons (cons import-handle-only #`ids) #`mapper))
        (syntax-error (syntax ids) "invalid 'only' identifier list")))
   ((_ (except import-set . ids) mapper)
    (if (identifier-list? #'ids)
-       #`(%import import-set ,(cons (cons import-handle-except #`ids) #`mapper))
+       #`(%import import-set #,(cons (cons import-handle-except #`ids) #`mapper))
        (syntax-error (syntax ids) "invalid 'except' identifier list")))
   ((_ (prefix import-set pfix) mapper)
-   #`(%import import-set ,(cons (cons import-handle-prefix #`pfix) #`mapper)))
+   #`(%import import-set #,(cons (cons import-handle-prefix #`pfix) #`mapper)))
   ((_ (prefix import-set . rest) mapper)
    (syntax-error (syntax rest) "invalid prefix clause in import"))
   ((_ (library libref) mapper)
-   #`(kawa.standard.ImportFromLibrary:instance libref ,(import-mapper (syntax-object->datum (syntax mapper)))))
+   #`(kawa.standard.ImportFromLibrary:instance libref #,(import-mapper (syntax-object->datum (syntax mapper)))))
   ((_ libref mapper)
-   #`(kawa.standard.ImportFromLibrary:instance libref ,(import-mapper (syntax-object->datum (syntax mapper))))))
+   #`(kawa.standard.ImportFromLibrary:instance libref #,(import-mapper (syntax-object->datum (syntax mapper))))))
 
 ;; LET-VALUES implementation from SRFI-11, by Lars T Hansen.
 ;; http://srfi.schemers.org/srfi-11/srfi-11.html
@@ -157,10 +159,10 @@
   (syntax-case form ()
     ((_ . cl)
      #`(gnu.expr.GenericProc:makeWithoutSorting
-	. ,(let loop ((clauses #'cl))
+	. #,(let loop ((clauses #'cl))
 	     (syntax-case clauses ()
 	       (((formals . body) . rest)
-		#`((lambda formals . body) . ,(loop #'rest)))
+		#`((lambda formals . body) . #,(loop #'rest)))
 	       (()
 		'())
 	       (rest
