@@ -345,7 +345,7 @@ public class repl extends Procedure0or1
 		      }
 		  }
 		getLanguageFromFilenameExtension(filename);
-		freader = InPort.openFile(fstream, filename);
+		freader = InPort.openFile(fstream, Path.valueOf(filename));
 		// FIXME adjust line number
 		setArgs(args, iArg+1);
 		checkInitFile();
@@ -682,7 +682,7 @@ public class repl extends Procedure0or1
 	    System.out.print("Kawa ");
 	    System.out.print(Version.getVersion());
 	    System.out.println();
-	    System.out.println("Copyright (C) 2011 Per Bothner");
+	    System.out.println("Copyright (C) 2012 Per Bothner");
 	    something_done = true;
 	  }
 	else if (arg.length () > 0 && arg.charAt(0) == '-')
@@ -794,7 +794,6 @@ public class repl extends Procedure0or1
             comp
               = language.parse(fstream, messages,
                                defaultParseOptions, minfo);
-            comp.getModule().classFor(comp);
             infos[i-iArg] = minfo;
             comps[i-iArg] = comp;
 
