@@ -2,20 +2,14 @@ package kawa.standard;
 import kawa.lang.*;
 import gnu.bytecode.Type;
 import gnu.bytecode.ClassType;
-import gnu.bytecode.ArrayType;
 import gnu.mapping.*;
-import gnu.math.IntNum;
 import gnu.expr.*;
 import java.util.*;
-import gnu.text.Lexer;
 import gnu.text.SourceMessages;
 import gnu.kawa.lispexpr.*;
 import gnu.lists.AbstractFormat;
 import gnu.kawa.functions.*;
-import gnu.kawa.reflect.ClassMethods;
-import gnu.kawa.reflect.StaticFieldLocation;
 import gnu.kawa.reflect.LazyType;
-import gnu.math.DFloNum;
 import gnu.kawa.servlet.HttpRequestContext;
 
 public class Scheme extends LispLanguage
@@ -155,6 +149,14 @@ public class Scheme extends LispLanguage
 
       defSntxStFld("lambda", "kawa.standard.SchemeCompilation", "lambda");
       defSntxStFld("$bracket-apply$", "gnu.kawa.lispexpr.BracketApply", "instance");
+      defSntxStFld("$xml-element$", "gnu.kawa.lispexpr.MakeXmlElement", "makeXml");
+      defProcStFld("$xml-attribute$", "gnu.kawa.xml.MakeAttribute", "makeAttributeS");
+      defProcStFld("$xml-text$", "gnu.kawa.xml.MakeText", "makeText");
+      defProcStFld("$xml-CDATA$", "gnu.kawa.xml.MakeCDATA", "makeCDATA");
+      defProcStFld("$xml-comment$", "gnu.kawa.xml.CommentConstructor", "commentConstructor");
+      defProcStFld("$xml-processing-instruction$", "gnu.kawa.xml.MakeProcInst", "makeProcInst");
+      defSntxStFld("$resolve-qname$", "gnu.kawa.lispexpr.ResolveNamespace",
+                   "resolveQName");
 
       //-- Section 4.1  -- complete
       defSntxStFld(LispLanguage.quote_sym, "kawa.lang.Quote", "plainQuote");
@@ -399,6 +401,10 @@ public class Scheme extends LispLanguage
       defProcStFld("vector-set!", "kawa.lib.vectors");
       defProcStFld("list->vector", "kawa.lib.vectors");
       defProcStFld("vector->list", "kawa.lib.vectors");
+      defProcStFld("vector->string", "kawa.lib.vectors");
+      defProcStFld("string->vector", "kawa.lib.vectors");
+      defProcStFld("vector-copy", "kawa.lib.vectors");
+      defProcStFld("vector-copy!", "kawa.lib.vectors");
       defProcStFld("vector-fill!", "kawa.lib.vectors");
       // Extension:
       defProcStFld("vector-append", "kawa.standard.vector_append", "vectorAppend");
@@ -421,17 +427,25 @@ public class Scheme extends LispLanguage
       defProcStFld("promise-set-exception!", "kawa.lib.misc");
       defProcStFld("promise-set-thunk!", "kawa.lib.misc");
 
-      //-- Section 6.10  -- complete
+      defProcStFld("call-with-port", "kawa.lib.ports");
       defProcStFld("call-with-input-file", "kawa.lib.ports");
       defProcStFld("call-with-output-file", "kawa.lib.ports");
       defProcStFld("input-port?", "kawa.lib.ports");
       defProcStFld("output-port?", "kawa.lib.ports");
+      defProcStFld("textual-port?", "kawa.lib.ports");
+      defProcStFld("binary-port?", "kawa.lib.ports");
+      defProcStFld("port?", "kawa.lib.ports");
+      defProcStFld("input-port-open?", "kawa.lib.ports");
+      defProcStFld("output-port-open?", "kawa.lib.ports");
       defProcStFld("current-input-port", "kawa.lib.ports");
       defProcStFld("current-output-port", "kawa.lib.ports");
       defProcStFld("with-input-from-file", "kawa.lib.ports");
       defProcStFld("with-output-to-file", "kawa.lib.ports");
       defProcStFld("open-input-file", "kawa.lib.ports");
+      defProcStFld("open-binary-input-file", "kawa.lib.ports");
       defProcStFld("open-output-file", "kawa.lib.ports");
+      defProcStFld("open-binary-output-file", "kawa.lib.ports");
+      defProcStFld("close-port", "kawa.lib.ports");
       defProcStFld("close-input-port", "kawa.lib.ports");
       defProcStFld("close-output-port", "kawa.lib.ports");
       defProcStFld("read", "kawa.lib.ports");
@@ -439,12 +453,24 @@ public class Scheme extends LispLanguage
       defProcStFld("read-char", "kawa.standard.readchar", "readChar");
       defProcStFld("peek-char", "kawa.standard.readchar", "peekChar");
       defProcStFld("eof-object?", "kawa.lib.ports");
+      defProcStFld("eof-object", "kawa.lib.ports");
+      defProcStFld("read-string", "kawa.lib.ports");
+      defProcStFld("read-u8", "kawa.lib.ports");
+      defProcStFld("peek-u8", "kawa.lib.ports");
+      defProcStFld("u8-ready", "kawa.lib.ports");
+      defProcStFld("read-bytevector", "kawa.lib.ports");
+      defProcStFld("read-bytevector!", "kawa.lib.ports");
       defProcStFld("char-ready?", "kawa.lib.ports");
       defProcStFld("write", "kawa.lib.ports");
+      defProcStFld("write-shared", "kawa.lib.ports");
       defProcStFld("write-with-shared-structure", "kawa.lib.ports");
+      defProcStFld("write-simple", "kawa.lib.ports");
       defProcStFld("display", "kawa.lib.ports");
       defProcStFld("print-as-xml", "gnu.xquery.lang.XQuery", "writeFormat");
       defProcStFld("write-char", "kawa.lib.ports");
+      defProcStFld("write-string", "kawa.lib.ports");
+      defProcStFld("write-u8", "kawa.lib.ports");
+      defProcStFld("write-bytevector", "kawa.lib.ports");
       defProcStFld("newline", "kawa.lib.ports");
       defProcStFld("load", "kawa.standard.load", "load");
       defProcStFld("load-relative", "kawa.standard.load", "loadRelative");
@@ -454,7 +480,11 @@ public class Scheme extends LispLanguage
       defProcStFld("open-input-string", "kawa.lib.ports");  // SRFI-6
       defProcStFld("open-output-string", "kawa.lib.ports");  // SRFI-6
       defProcStFld("get-output-string", "kawa.lib.ports");  // SRFI-6
+      defProcStFld("open-input-bytevector", "kawa.lib.ports");
+      defProcStFld("open-output-bytevector", "kawa.lib.ports");
+      defProcStFld("get-output-bytevector", "kawa.lib.ports");
       defProcStFld("call-with-output-string", "kawa.lib.ports"); // Extension
+      defProcStFld("flush-output-port", "kawa.lib.ports");  // R6RS/R7RS
       defProcStFld("force-output", "kawa.lib.ports");  // Extension
 
       defProcStFld("port-line", "kawa.lib.ports");
@@ -729,8 +759,8 @@ public class Scheme extends LispLanguage
       defProcStFld("format", "gnu.kawa.functions.Format");
       defProcStFld("parse-format", "gnu.kawa.functions.ParseFormat", "parseFormat");
 
-      defProcStFld("make-element", "gnu.kawa.xml.MakeElement", "makeElement");
-      defProcStFld("make-attribute", "gnu.kawa.xml.MakeAttribute", "makeAttribute");
+      defProcStFld("make-element", "gnu.kawa.xml.MakeElement", "makeElementS");
+      defProcStFld("make-attribute", "gnu.kawa.xml.MakeAttribute", "makeAttributeS");
       defProcStFld("map-values", "gnu.kawa.functions.ValuesMap", "valuesMap");
       defProcStFld("children", "gnu.kawa.xml.Children", "children");
       defProcStFld("attributes", "gnu.kawa.xml.Attributes");
@@ -793,13 +823,15 @@ public class Scheme extends LispLanguage
 
       defAliasStFld("bytevector", "gnu.kawa.lispexpr.LangObjType",
                     "u8vectorType");
-      defProcStFld("make-bytevector", "kawa.lib.bytevector");
-      defProcStFld("bytevector-length", "kawa.lib.bytevector");
-      defProcStFld("bytevector-u8-ref", "kawa.lib.bytevector");
-      defProcStFld("bytevector-u8-set!", "kawa.lib.bytevector");
-      defProcStFld("bytevector-copy", "kawa.lib.bytevector");
-      defProcStFld("utf8->string", "kawa.lib.bytevector");
-      defProcStFld("string->utf8", "kawa.lib.bytevector");
+      defProcStFld("make-bytevector", "kawa.lib.bytevectors");
+      defProcStFld("bytevector-length", "kawa.lib.bytevectors");
+      defProcStFld("bytevector-u8-ref", "kawa.lib.bytevectors");
+      defProcStFld("bytevector-u8-set!", "kawa.lib.bytevectors");
+      defProcStFld("bytevector-copy", "kawa.lib.bytevectors");
+      defProcStFld("bytevector-copy!", "kawa.lib.bytevectors");
+      defProcStFld("bytevector-append", "kawa.lib.bytevectors");
+      defProcStFld("utf8->string", "kawa.lib.bytevectors");
+      defProcStFld("string->utf8", "kawa.lib.bytevectors");
 
       defSntxStFld("cut", "gnu.kawa.slib.cut");
       defSntxStFld("cute", "gnu.kawa.slib.cut");
@@ -990,134 +1022,25 @@ public class Scheme extends LispLanguage
     return getInstance().getTypeFor(exp);
   }
 
-  static HashMap<String,Type> types;
-  static HashMap<Type,String> typeToStringMap;
+    private HashMap<String,Type> types;
+    private HashMap<Type,String> typeToStringMap;
 
-  static synchronized HashMap<String,Type> getTypeMap ()
-  {
-    if (types == null)
-      {
-	booleanType
-	  = new LangPrimType(Type.booleanType, Scheme.getInstance());
-	types = new HashMap<String,Type> ();
-	types.put ("void", LangPrimType.voidType);
-	types.put ("int", LangPrimType.intType);
-	types.put ("char", LangPrimType.charType);
-	types.put ("boolean", booleanType);
-	types.put ("byte", LangPrimType.byteType);
-	types.put ("short", LangPrimType.shortType);
-	types.put ("long", LangPrimType.longType);
-	types.put ("float", LangPrimType.floatType);
-	types.put ("double", LangPrimType.doubleType);
-	types.put ("never-returns", Type.neverReturnsType);
-
-	types.put ("Object", Type.objectType);
-	types.put ("String", Type.toStringType);
-
-	types.put ("object", Type.objectType);
-	types.put ("number", LangObjType.numericType);
-	types.put ("quantity", ClassType.make("gnu.math.Quantity"));
-	types.put ("complex", ClassType.make("gnu.math.Complex"));
-	types.put ("real", LangObjType.realType);
-	types.put ("rational", LangObjType.rationalType);
-	types.put ("integer", LangObjType.integerType);
-	types.put ("symbol", ClassType.make("gnu.mapping.Symbol"));
-	types.put ("namespace", ClassType.make("gnu.mapping.Namespace"));
-	types.put ("keyword", ClassType.make("gnu.expr.Keyword"));
-	types.put ("pair", ClassType.make("gnu.lists.Pair"));
-	types.put ("pair-with-position",
-		   ClassType.make("gnu.lists.PairWithPosition"));
-	types.put ("constant-string", ClassType.make("java.lang.String"));
-	types.put ("abstract-string", ClassType.make("gnu.lists.CharSeq"));
-	types.put ("character", ClassType.make("gnu.text.Char"));
-	types.put ("vector", LangObjType.vectorType);
-	types.put ("string", LangObjType.stringType);
-        types.put ("empty-list", ClassType.make("gnu.lists.EmptyList"));
-	types.put ("list", LangObjType.listType);
-	types.put ("function", ClassType.make("gnu.mapping.Procedure"));
-	types.put ("procedure", LangObjType.procedureType);
-	types.put ("input-port", ClassType.make("gnu.mapping.InPort"));
-	types.put ("output-port", ClassType.make("gnu.mapping.OutPort"));
-	types.put ("string-output-port",
-                   ClassType.make("gnu.mapping.CharArrayOutPort"));
-	types.put ("string-input-port",
-                   ClassType.make("gnu.mapping.CharArrayInPort"));
-	types.put ("record", ClassType.make("kawa.lang.Record"));
-	types.put ("type", LangObjType.typeType);
-	types.put ("class-type", LangObjType.typeClassType);
-	types.put ("class", LangObjType.typeClass);
-	types.put ("promise", LangObjType.promiseType);
-
-        for (int i = uniformVectorTags.length;  --i >= 0; )
-          {
-            String tag = uniformVectorTags[i];
-            String cname = "gnu.lists."+tag.toUpperCase()+"Vector";
-            types.put(tag+"vector", ClassType.make(cname));
-          }
-
-        types.put ("document", ClassType.make("gnu.kawa.xml.KDocument"));
-        types.put ("readtable", ClassType.make("gnu.kawa.lispexpr.ReadTable"));
-      }
-    return types;
-  }
-
-  public static Type getNamedType (String name)
-  {
-    getTypeMap();
-    Type type = (Type) types.get(name);
-    if (type == null
-	&& (name.startsWith("elisp:") || name.startsWith("clisp:")))
-      {
-	int colon = name.indexOf(':');
-	Class clas = getNamedType(name.substring(colon+1)).getReflectClass();
-	String lang = name.substring(0,colon);
-	Language interp = Language.getInstance(lang);
-	if (interp == null)
-	    throw new RuntimeException("unknown type '" + name
-				       + "' - unknown language '"
-				       + lang + '\'');
-	type = interp.getTypeFor(clas);
-	if (type != null)
-	  types.put(name, type);
-      }
-    return type;
-  }
-
-  public Type getTypeFor (Class clas)
-  {
-    String name = clas.getName();
-    if (clas.isPrimitive())
-      return getNamedType(name);
-    if ("java.lang.String".equals(name))
-      return Type.toStringType;
-    if ("gnu.math.IntNum".equals(name))
-      return LangObjType.integerType;
-    if ("gnu.math.DFloNum".equals(name))
-      return LangObjType.dflonumType;
-    if ("gnu.math.RatNum".equals(name))
-      return LangObjType.rationalType;
-    if ("gnu.math.RealNum".equals(name))
-      return LangObjType.realType;
-    if ("gnu.math.Numeric".equals(name))
-      return LangObjType.numericType;
-    if ("gnu.lists.FVector".equals(name))
-      return LangObjType.vectorType;
-    if ("gnu.lists.LList".equals(name))
-      return LangObjType.listType;
-    if ("gnu.text.Path".equals(name))
-      return LangObjType.pathType;
-    if ("gnu.text.URIPath".equals(name))
-      return LangObjType.URIType;
-    if ("gnu.text.FilePath".equals(name))
-      return LangObjType.filepathType;
-    if ("java.lang.Class".equals(name))
-      return LangObjType.typeClass;
-    if ("gnu.bytecode.Type".equals(name))
-      return LangObjType.typeType;
-    if ("gnu.bytecode.ClassType".equals(name))
-      return LangObjType.typeClassType;
-    return Type.make(clas);
-  }
+    @Override
+    protected synchronized HashMap<String, Type> getTypeMap() {
+        if (types == null) {
+            booleanType = new LangPrimType(Type.booleanType,
+                                           Scheme.getInstance());
+            types = new HashMap<String, Type>(128); // Bit more wiggle room
+            types.put("boolean", booleanType);
+            types.putAll(super.getTypeMap());
+            for (int i = uniformVectorTags.length; --i >= 0;) {
+                String tag = uniformVectorTags[i];
+                String cname = "gnu.lists." + tag.toUpperCase() + "Vector";
+                types.put(tag + "vector", ClassType.make(cname));
+            }
+        }
+        return types;
+    }
 
   public String formatType (Type type)
   {
@@ -1147,31 +1070,7 @@ public class Scheme extends LispLanguage
     return super.formatType(type);
   }
 
-  public static Type string2Type (String name)
-  {
-    Type t;
-    if (name.endsWith("[]"))
-      {
-	t = string2Type(name.substring(0, name.length()-2));
-	if (t != null)
-	  t = ArrayType.make(t);
-      }
-    else
-      t = getNamedType (name);
-    if (t != null)
-      return t;
-    t = Language.string2Type(name);
-    if (t != null)
-      types.put (name, t);
-    return t;
-  }
-
-  public Type getTypeFor(String name)
-  {
-    return string2Type(name);
-  }
-
-  /** Convert expression to a Type.
+ /** Convert expression to a Type.
    * Allow "TYPE" or 'TYPE or <TYPE>.
    */
   public static Type exp2Type (Expression exp)
