@@ -1,5 +1,6 @@
 package kawa.standard;
 import gnu.mapping.*;
+import gnu.kawa.lispexpr.LangObjType;
 
 public class call_with_values extends Procedure2
 {
@@ -18,15 +19,16 @@ public class call_with_values extends Procedure2
 
   public Object apply2 (Object producer, Object consumer) throws Throwable
   {
-    return callWithValues((Procedure) producer, (Procedure) consumer);
+    return callWithValues(LangObjType.coerceToProcedure(producer),
+                          LangObjType.coerceToProcedure(consumer));
   }
 
   public void apply (CallContext ctx) throws Throwable
   {
     Procedure.checkArgCount(this, 2);
     Object[] args = ctx.getArgs();
-    Object values = ((Procedure) args[0]).apply0 ();
-    Procedure consumer = (Procedure) args[1];
+    Object values = LangObjType.coerceToProcedure(args[0]).apply0();
+    Procedure consumer = LangObjType.coerceToProcedure(args[1]);
     if (values instanceof Values)
       {
 	args = ((Values) values).getValues();
