@@ -7,6 +7,9 @@ import gnu.expr.*;
 
 public class constant_fold extends Syntax
 {
+  public static final constant_fold constant_fold = new constant_fold();
+  static { constant_fold.setName("constant-fold"); }
+
   static Object checkConstant(Expression exp, Translator tr)
   {
     if (exp instanceof QuoteExp)
@@ -14,8 +17,11 @@ public class constant_fold extends Syntax
     if (exp instanceof ReferenceExp)
       {
 	ReferenceExp rexp = (ReferenceExp) exp;
-	if (rexp.getBinding() == null)
+        Declaration decl = rexp.getBinding();
+	if (decl == null || decl.getFlag(Declaration.IS_UNKNOWN))
 	  return Environment.user().get(rexp.getName(), null);
+        else
+          return Declaration.followAliases(decl).getConstantValue();
       }
     return null;
   }
@@ -29,6 +35,11 @@ public class constant_fold extends Syntax
     Object func = checkConstant(aexp.getFunction(), tr);
     if (! (func instanceof Procedure))
       return exp;
+
+    // Not quite the same - checkConstant also looks up name in Environment,
+    // which seems a bit too dangerous for inlineIfConstant.  FIXME.
+    // return aexp.inlineIfConstant((Procedure) func, tr.getMessages());
+
     Expression[] args = aexp.getArgs();
     int i = args.length;
     Object[] vals = new Object[i];
