@@ -305,6 +305,8 @@ public class object extends Syntax
 	  { // Method declaration.
 	    Pair mpair = (Pair) pair_car;
 	    Object mname = mpair.getCar();
+            while (mname instanceof SyntaxForm)
+              mname = ((SyntaxForm) mname).getDatum();
 	    if (! (mname instanceof String)
 		&& ! (mname instanceof Symbol))
 	      {
@@ -346,11 +348,7 @@ public class object extends Syntax
             && (classNameSpecifier = classNameVal.toString()).length() > 0)
           oexp.classNameSpecifier = classNameSpecifier;
         else
-          {
-            Object savedPos = tr.pushPositionOf(classNamePair);
-            tr.error('e', "class-name specifier must be a non-empty string literal");
-            tr.popPositionOf(savedPos);
-          }
+            tr.errorWithPosition("class-name specifier must be a non-empty string literal", classNamePair);
       }
 
     Object[] result = {
