@@ -14,6 +14,39 @@ public class syntax extends kawa.lang.Quote
     super(name, isQuasi);
   }
 
+    protected boolean matchesUnquote(Pair pair, SyntaxForm syntax,
+                                     Translator tr) {
+        Object form = pair.getCar();
+        if (tr.matches(form, syntax, "unsyntax"))
+            return true;
+        if (tr.matches(form, syntax, "unquote")) {
+            tr.error('w',
+                "unquote inside quasisyntax is deprecated - use unsyntax or #,",
+                pair instanceof PairWithPosition ? (PairWithPosition)pair : tr);
+            return true;
+        }
+        return false;
+    }
+
+    protected boolean matchesUnquoteSplicing(Pair pair, SyntaxForm syntax,
+                                     Translator tr) {
+        Object form = pair.getCar();
+        if (tr.matches(form, syntax, "unsyntax-splicing"))
+            return true;
+        if (tr.matches(form, syntax, "unquote-splicing")) {
+            tr.error('w',
+                "unquote-splicing inside quasisyntax is deprecated - use unsyntax-splicing or #@,",
+                pair instanceof PairWithPosition ? (PairWithPosition)pair : tr);
+            return true;
+        }
+        return false;
+    }
+
+    protected boolean matchesQuasiQuote(Object form, SyntaxForm syntax,
+                                     Translator tr) {
+        return tr.matches(form, syntax, "quasisyntax");
+    }
+
   protected boolean expandColonForms ()
   {
     return false;
