@@ -1,4 +1,44 @@
-(test-begin "libs" 226)
+(test-begin "libs" 254)
+
+(test-begin "vectors")
+(test-equal '(dah dah didah)
+            (vector->list '#(dah dah didah)))
+(test-equal '(dah)
+             (vector->list '#(dah dah didah) 1 2))
+(test-equal #(dididit dah)
+            (list->vector '(dididit dah)))
+
+(test-equal #(#\A #\B #\C)
+            (string->vector "ABC"))
+(test-equal #(#\B #\C)
+            (string->vector "ABCDE" 1 3))
+(test-equal "123"
+            (vector->string #(#\1 #\2 #\3)))
+(test-equal "345"
+            (vector->string #(#\1 #\2 #\3 #\4 #\5) 2))
+(test-equal "34"
+            (vector->string #(#\1 #\2 #\3 #\4 #\5) 2 4))
+ 
+(let* ((a #(1 8 2 8))
+       (b (vector-copy a)))
+  (vector-set! b 0 3)
+  (test-equal #(3 8 2 8) b)
+  (test-equal #(8 2) (vector-copy b 1 3)))
+
+(test-equal #(a b c d e f)
+            (vector-append #(a b c) #(d e f)))
+
+(let* ((a (vector 1 2 3 4 5))
+       (b (vector 10 20 30 40 50)))
+  (vector-copy! b 1 a 0 2)
+  (test-equal #(10 1 2 40 50) b))
+
+(test-equal #(1 2 smash smash 5)
+            (let ()
+              (define a (vector 1 2 3 4 5))
+              (vector-fill! a 'smash 2 4)
+              a))
+(test-end)
 
 (test-begin "bytevectors") ;; Some bytevector tests
 (define bytes1 (bytevector #u8(#xCE #xBB)))
@@ -8,6 +48,19 @@
 (test-equal 2 (bytevector-length bytes1))
 (test-equal 187 (bytevector-u8-ref bytes1 1))
 (test-equal #f (bytevector? lambda-string))
+(let ((bv (bytevector 1 2 3 4)))
+  (bytevector-u8-set! bv 1 3)
+  (test-equal #u8(1 3 3 4) bv))
+(let ((a #u8(1 2 3 4 5)))
+  (test-equal #u8(3 4) (bytevector-copy a 2 4)))
+(let ((a (bytevector 1 2 3 4 5))
+      (b (bytevector 10 20 30 40 50)))
+  (bytevector-copy! b 1 a 0 2)
+  (test-equal #u8(10 1 2 40 50) b))
+(test-equal #u8(0 1 2 3 4 5)
+            (bytevector-append #u8(0 1 2) #u8(3 4 5)))
+(test-equal "A" (utf8->string #u8(#x41)))
+(test-equal #u8(#xCE #xBB) (string->utf8 "λ"))
 (test-equal bytes1 (string->utf8 lambda-string))
 (test-equal lambda-string (utf8->string bytes1))
 (test-end)
@@ -148,6 +201,26 @@
 (test-equal '(as <java.lang.String> (quote a))
       (expand '(as String 'a)))
 
+(test-equal '(lambda (a b c) #f)
+	    (expand '(lambda (a b c) #f)))
+(test-equal '(lambda (#!rest r) #f)
+	    (expand '(lambda r #f)))
+(test-equal '(lambda (#!rest r) #f)
+	    (expand '(lambda (#!rest r) #f)))
+(test-equal '(lambda (a b c #!rest r) #f)
+	    (expand '(lambda (a b c #!rest r) #f)))
+(test-equal '(lambda (#!optional d e f) #f)
+	    (expand '(lambda (#!optional d e f) #f)))
+(test-equal '(lambda (a b c #!optional d e f) #f)
+	    (expand '(lambda (a b c #!optional d e f) #f)))
+(test-equal '(lambda (a b c #!optional d e f #!rest r) #f)
+	    (expand '(lambda (a b c #!optional d e f #!rest r) #f)))
+(test-equal '(lambda (a b c #!rest r #!key d e f ) #f)
+	    (expand '(lambda (a b c #!rest r #!key d e f ) #f)))
+(test-equal '(lambda (a b c #!optional d e f #!rest r #!key g i j ) #f)
+	    (expand '(lambda (a b c #!optional d e f #!rest r #!key g i j)
+		      #f)))
+
 (import (srfi :41 streams))
 
 (define strm123
