@@ -2,9 +2,9 @@
 
 (require <module1>)
 
-(module-export dvar-test-1 factorial-4 check-fluid-let *VAR*
-	       namespace-syntax-call list-length-4 my-compare
-	       test3-import1 get3-mod0-v2 set3-mod0-v2
+(module-export dvar-test-1 factorial-4 check-fluid-let *VAR* check-thunk
+	       namespace-syntax-call list-length-4 my-compare test3-import2
+	       test3-import1 get3-mod0-v2 set3-mod0-v2 counter-test-result
 	       pa-new pa-getter pa-setter pa-length iarr-set mB test1-import0)
 
 (define (get3-mod0-v1) :: <object> mod0-v1)
@@ -82,3 +82,17 @@
        ((fn o)
         (simpleAux:init o)
         (list (slot-ref o 'x) name))))))
+
+;; Andre van Tonder <andre@het.brown.edu> example in posting 2011-04-19.
+(define counter-test-result
+  (let* ((a (counter-macro))
+         (b (counter)))
+    (list a b)))
+
+;; Test for Savannah bug #34004: Nullpointer exception in compiler
+(define (call-thunk thunk::procedure)
+  (thunk))
+(define (call-call-thunk x)
+  (call-thunk (lambda () x)))
+(define (check-thunk)
+  (call-call-thunk 1))
